@@ -7,10 +7,8 @@
 #  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
 #
 import asyncio
+import itertools
 from logging import getLogger
-from typing import Any
-
-from .recursive import get_recursive
 
 logger = getLogger(__name__)
 
@@ -40,9 +38,19 @@ class AsyncInitable:
         self.async_init_finished__event.set()
 
     async def _async_init(self) -> None:
-        pass
+        await self._async_init_attrs()
 
-    @staticmethod
-    async def async_init_recursive(target: Any) -> None:
-        async_initables = await get_recursive(target, condition=lambda x: isinstance(x, AsyncInitable))
-        await asyncio.gather(*[x.async_init() for x in async_initables if not x.async_init_scheduled__event.is_set()])
+    async def _async_init_attrs(self) -> None:
+        async with asyncio.TaskGroup() as tg:
+            for attribute_name in vars(self):
+                attribute = getattr(self, attribute_name)
+                if isinstance(attribute, AsyncInitable) and not attribute.async_init_scheduled__event.is_set():
+                    tg.create_task(attribute.async_init())
+                if isinstance(attribute, (list, set)):
+                    for val in attribute:
+                        if isinstance(val, AsyncInitable) and not val.async_init_scheduled__event.is_set():
+                            tg.create_task(val.async_init())
+                if isinstance(attribute, dict):
+                    for val in itertools.chain(attribute.keys(), attribute.values()):
+                        if isinstance(val, AsyncInitable) and not val.async_init_scheduled__event.is_set():
+                            tg.create_task(val.async_init())
